@@ -1,31 +1,26 @@
 /*
- *   Copyright 2014 Martin Gräßlin <mgraesslin@kde.org>
- *
- *   This library is free software; you can redistribute it and/or
- *   modify it under the terms of the GNU Lesser General Public
- *   License as published by the Free Software Foundation; either
- *   version 2.1 of the License, or (at your option) version 3, or any
- *   later version accepted by the membership of KDE e.V. (or its
- *   successor approved by the membership of KDE e.V.), which shall
- *   act as a proxy defined in Section 6 of version 3 of the license.
- *
- *   This library is distributed in the hope that it will be useful,
- *   but WITHOUT ANY WARRANTY; without even the implied warranty of
- *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- *   Lesser General Public License for more details.
- *
- *   You should have received a copy of the GNU Lesser General Public
- *   License along with this library.  If not, see <http://www.gnu.org/licenses/>.
- */
+    SPDX-FileCopyrightText: 2014 Martin Gräßlin <mgraesslin@kde.org>
+
+    SPDX-License-Identifier: LGPL-2.1-only OR LGPL-3.0-only OR LicenseRef-KDE-Accepted-LGPL
+*/
+
 #include "kwindowinfo.h"
 #include "kwindowsystem.h"
-#include "netwm.h"
+#include "kx11extras.h"
 #include "nettesthelper.h"
+#include "netwm.h"
 
-#include <qtest_widgets.h>
 #include <QScreen>
 #include <QSignalSpy>
+#include <QSysInfo>
+#if QT_VERSION >= QT_VERSION_CHECK(6, 0, 0)
+#include <private/qtx11extras_p.h>
+#else
 #include <QX11Info>
+#endif
+#include <qtest_widgets.h>
+
+#include <xcb/xcb_icccm.h>
 
 #include <unistd.h>
 
@@ -38,7 +33,6 @@ Q_DECLARE_METATYPE(NET::WindowTypes)
 Q_DECLARE_METATYPE(NET::Properties)
 Q_DECLARE_METATYPE(NET::Properties2)
 
-
 class KWindowInfoX11Test : public QObject
 {
     Q_OBJECT
@@ -49,7 +43,9 @@ private Q_SLOTS:
 
     void testState_data();
     void testState();
+#if KWINDOWSYSTEM_BUILD_DEPRECATED_SINCE(5, 101)
     void testDemandsAttention();
+#endif
     void testMinimized();
     void testMappingState();
     void testWindowType_data();
@@ -76,7 +72,7 @@ private:
     bool waitForWindow(QSignalSpy &spy, WId winId, NET::Properties property, NET::Properties2 properties2 = NET::Properties2()) const;
     bool verifyMinimized(WId window) const;
 
-    QScopedPointer<QWidget> window;
+    std::unique_ptr<QWidget> window;
 };
 
 void KWindowInfoX11Test::initTestCase()
@@ -86,7 +82,7 @@ void KWindowInfoX11Test::initTestCase()
     qRegisterMetaType<NET::Properties2>();
 }
 
-bool KWindowInfoX11Test::waitForWindow(QSignalSpy& spy, WId winId, NET::Properties property, NET::Properties2 property2) const
+bool KWindowInfoX11Test::waitForWindow(QSignalSpy &spy, WId winId, NET::Properties property, NET::Properties2 property2) const
 {
     // we need to wait, window manager has to react and update the property.
     bool foundOurWindow = false;
@@ -130,13 +126,13 @@ void KWindowInfoX11Test::init()
 {
     // create the window and ensure it has been managed
     window.reset(new QWidget());
-    showWidget(window.data());
+    showWidget(window.get());
 }
 
 void KWindowInfoX11Test::showWidget(QWidget *window)
 {
     qRegisterMetaType<WId>("WId");
-    QSignalSpy spy(KWindowSystem::self(), SIGNAL(windowAdded(WId)));
+    QSignalSpy spy(KX11Extras::self(), &KX11Extras::windowAdded);
     window->show();
     bool foundOurWindow = false;
     for (int i = 0; i < 50; ++i) {
@@ -163,9 +159,9 @@ void KWindowInfoX11Test::showWidget(QWidget *window)
 void KWindowInfoX11Test::cleanup()
 {
     // we hide the window and wait till it is gone so that we have a clean state in next test
-    if (!window.isNull() && window->isVisible()) {
+    if (window && window->isVisible()) {
         WId id = window->winId();
-        QSignalSpy spy(KWindowSystem::self(), SIGNAL(windowRemoved(WId)));
+        QSignalSpy spy(KX11Extras::self(), &KX11Extras::windowRemoved);
         window->hide();
         bool foundOurWindow = false;
         for (int i = 0; i < 50; ++i) {
@@ -192,14 +188,19 @@ void KWindowInfoX11Test::testState_data()
 {
     QTest::addColumn<NET::States>("state");
 
-    QTest::newRow("max")         << NET::States(NET::Max);
-    QTest::newRow("maxHoriz")    << NET::States(NET::MaxHoriz);
-    QTest::newRow("shaded")      << NET::States(NET::Shaded);
+    QTest::newRow("max") << NET::States(NET::Max);
+    QTest::newRow("maxHoriz") << NET::States(NET::MaxHoriz);
+    QTest::newRow("shaded") << NET::States(NET::Shaded);
     QTest::newRow("skipTaskbar") << NET::States(NET::SkipTaskbar);
-    QTest::newRow("skipPager")   << NET::States(NET::SkipPager);
-    QTest::newRow("keep above")  << NET::States(NET::KeepAbove);
-    QTest::newRow("keep below")  << NET::States(NET::KeepBelow);
-    QTest::newRow("fullscreen")  << NET::States(NET::FullScreen);
+    QTest::newRow("skipPager") << NET::States(NET::SkipPager);
+    QTest::newRow("keep above") << NET::States(NET::KeepAbove);
+    QTest::newRow("keep below") << NET::States(NET::KeepBelow);
+    QTest::newRow("fullscreen") << NET::States(NET::FullScreen);
+
+    NETRootInfo info(QX11Info::connection(), NET::Supported);
+    if (info.isSupported(NET::SkipSwitcher)) {
+        QTest::newRow("skipSwitcher") << NET::States(NET::SkipSwitcher);
+    }
 
     // NOTE: modal, sticky and hidden cannot be tested with this variant
     // demands attention is not tested as that's already part of the first run adjustments
@@ -217,7 +218,7 @@ void KWindowInfoX11Test::testState()
         QVERIFY(!info.hasState(NET::States(1 << i)));
     }
 
-    QSignalSpy spy(KWindowSystem::self(), static_cast<void (KWindowSystem::*)(WId,NET::Properties,NET::Properties2)>(&KWindowSystem::windowChanged));
+    QSignalSpy spy(KX11Extras::self(), &KX11Extras::windowChanged);
     QVERIFY(spy.isValid());
     // now we have a clean window and can do fun stuff
     KWindowSystem::setState(window->winId(), state);
@@ -226,30 +227,31 @@ void KWindowInfoX11Test::testState()
 
     KWindowInfo info3(window->winId(), NET::WMState);
     QVERIFY(info3.valid());
-    QCOMPARE(info3.state(), state);
+    QCOMPARE(int(info3.state()), int(state));
     QVERIFY(info3.hasState(state));
 }
 
 // This struct is defined here to avoid a dependency on xcb-icccm
 struct kde_wm_hints {
-    uint32_t      flags;
-    uint32_t      input;
-    int32_t       initial_state;
-    xcb_pixmap_t  icon_pixmap;
-    xcb_window_t  icon_window;
-    int32_t       icon_x;
-    int32_t       icon_y;
-    xcb_pixmap_t  icon_mask;
-    xcb_window_t  window_group;
+    uint32_t flags;
+    uint32_t input;
+    int32_t initial_state;
+    xcb_pixmap_t icon_pixmap;
+    xcb_window_t icon_window;
+    int32_t icon_x;
+    int32_t icon_y;
+    xcb_pixmap_t icon_mask;
+    xcb_window_t window_group;
 };
 
+#if KWINDOWSYSTEM_BUILD_DEPRECATED_SINCE(5, 101)
 void KWindowInfoX11Test::testDemandsAttention()
 {
-    QSignalSpy activeWindowSpy(KWindowSystem::self(), SIGNAL(activeWindowChanged(WId)));
+    QSignalSpy activeWindowSpy(KX11Extras::self(), &KX11Extras::activeWindowChanged);
     QVERIFY(activeWindowSpy.isValid());
-    if (KWindowSystem::activeWindow() != window->winId()) {
+    if (KX11Extras::activeWindow() != window->winId()) {
         // we force activate as KWin's focus stealing prevention might kick in
-        KWindowSystem::forceActiveWindow(window->winId());
+        KX11Extras::forceActiveWindow(window->winId());
         QVERIFY(activeWindowSpy.wait());
         QCOMPARE(activeWindowSpy.first().first().toULongLong(), window->winId());
         activeWindowSpy.clear();
@@ -258,7 +260,7 @@ void KWindowInfoX11Test::testDemandsAttention()
     // need a second window for proper interaction
     QWidget win2;
     showWidget(&win2);
-    KWindowSystem::forceActiveWindow(win2.winId());
+    KX11Extras::forceActiveWindow(win2.winId());
     if (activeWindowSpy.isEmpty()) {
         QVERIFY(activeWindowSpy.wait());
     }
@@ -267,7 +269,7 @@ void KWindowInfoX11Test::testDemandsAttention()
     QVERIFY(info.valid());
     QVERIFY(!info.hasState(NET::DemandsAttention));
 
-    QSignalSpy spy(KWindowSystem::self(), static_cast<void (KWindowSystem::*)(WId,NET::Properties,NET::Properties2)>(&KWindowSystem::windowChanged));
+    QSignalSpy spy(KX11Extras::self(), &KX11Extras::windowChanged);
     QVERIFY(spy.isValid());
     // now we have a clean window and can do fun stuff
     KWindowSystem::demandAttention(window->winId());
@@ -281,7 +283,7 @@ void KWindowInfoX11Test::testDemandsAttention()
 
     // now activate win1, that should remove demands attention
     spy.clear();
-    KWindowSystem::forceActiveWindow(window->winId());
+    KX11Extras::forceActiveWindow(window->winId());
     QTest::qWait(200);
     QVERIFY(waitForWindow(spy, window->winId(), NET::WMState));
 
@@ -319,8 +321,7 @@ void KWindowInfoX11Test::testDemandsAttention()
     hints.icon_x = 0;
     hints.icon_y = 0;
     hints.initial_state = 0;
-    xcb_change_property(QX11Info::connection(), XCB_PROP_MODE_REPLACE, win2.winId(),
-                        XCB_ATOM_WM_HINTS, XCB_ATOM_WM_HINTS, 32, 9, &hints);
+    xcb_change_property(QX11Info::connection(), XCB_PROP_MODE_REPLACE, win2.winId(), XCB_ATOM_WM_HINTS, XCB_ATOM_WM_HINTS, 32, 9, &hints);
     xcb_flush(QX11Info::connection());
 
     // window managers map urgency to demands attention
@@ -335,14 +336,14 @@ void KWindowInfoX11Test::testDemandsAttention()
 
     // remove urgency again
     hints.flags = 0;
-    xcb_change_property(QX11Info::connection(), XCB_PROP_MODE_REPLACE, win2.winId(),
-                        XCB_ATOM_WM_HINTS, XCB_ATOM_WM_HINTS, 32, 9, &hints);
+    xcb_change_property(QX11Info::connection(), XCB_PROP_MODE_REPLACE, win2.winId(), XCB_ATOM_WM_HINTS, XCB_ATOM_WM_HINTS, 32, 9, &hints);
     xcb_flush(QX11Info::connection());
     // TODO: test whether it gets removed again. At least KWin does not remove demands attention
 
     // prevent openbox crash (see https://bugzilla.icculus.org/show_bug.cgi?id=6315 )
     QTest::qWait(600);
 }
+#endif
 
 void KWindowInfoX11Test::testMinimized()
 {
@@ -389,6 +390,7 @@ void KWindowInfoX11Test::testWindowType_data()
     QTest::addColumn<NET::WindowType>("type");
     QTest::addColumn<NET::WindowType>("expectedType");
 
+    // clang-format off
     QTest::newRow("desktop")            << NET::DesktopMask      << NET::Desktop      << NET::Desktop;
     QTest::newRow("dock")               << NET::DockMask         << NET::Dock         << NET::Dock;
     QTest::newRow("toolbar")            << NET::ToolbarMask      << NET::Toolbar      << NET::Toolbar;
@@ -410,6 +412,8 @@ void KWindowInfoX11Test::testWindowType_data()
     QTest::newRow("ComboBox")           << NET::ComboBoxMask     << NET::ComboBox     << NET::ComboBox;
     QTest::newRow("DNDIcon")            << NET::DNDIconMask      << NET::DNDIcon      << NET::DNDIcon;
     QTest::newRow("OnScreenDisplay")    << NET::OnScreenDisplayMask << NET::OnScreenDisplay << NET::OnScreenDisplay;
+    QTest::newRow("CriticalNotification") << NET::CriticalNotificationMask << NET::CriticalNotification << NET::CriticalNotification;
+    QTest::newRow("AppletPopup")        << NET::AppletPopupMask  << NET::AppletPopup  << NET::AppletPopup;
 
     // incorrect masks
     QTest::newRow("desktop-unknown")      << NET::NormalMask << NET::Desktop      << NET::Unknown;
@@ -428,6 +432,9 @@ void KWindowInfoX11Test::testWindowType_data()
     QTest::newRow("ComboBox-unknown")     << NET::NormalMask << NET::ComboBox     << NET::Unknown;
     QTest::newRow("DNDIcon-unknown")      << NET::NormalMask << NET::DNDIcon      << NET::Unknown;
     QTest::newRow("OnScreenDisplay-unknown") << NET::NormalMask << NET::OnScreenDisplay << NET::Unknown;
+    QTest::newRow("CriticalNotification-unknown") << NET::NormalMask << NET::CriticalNotification << NET::Unknown;
+    QTest::newRow("AppletPopup-unknown")  << NET::NormalMask << NET::AppletPopup  << NET::Unknown;
+    // clang-format on
 }
 
 void KWindowInfoX11Test::testWindowType()
@@ -448,15 +455,15 @@ void KWindowInfoX11Test::testWindowType()
 
 void KWindowInfoX11Test::testDesktop()
 {
-    if (KWindowSystem::numberOfDesktops() < 2) {
+    if (KX11Extras::numberOfDesktops() < 2) {
         QSKIP("We need at least two virtual desktops to perform proper virtual desktop testing");
     }
     KWindowInfo info(window->winId(), NET::WMDesktop);
     QVERIFY(info.isOnCurrentDesktop());
     QVERIFY(!info.onAllDesktops());
-    QCOMPARE(info.desktop(), KWindowSystem::currentDesktop());
-    for (int i = 1; i < KWindowSystem::numberOfDesktops(); i++) {
-        if (i == KWindowSystem::currentDesktop()) {
+    QCOMPARE(info.desktop(), KX11Extras::currentDesktop());
+    for (int i = 1; i < KX11Extras::numberOfDesktops(); i++) {
+        if (i == KX11Extras::currentDesktop()) {
             QVERIFY(info.isOnDesktop(i));
         } else {
             QVERIFY(!info.isOnDesktop(i));
@@ -464,22 +471,22 @@ void KWindowInfoX11Test::testDesktop()
     }
 
     // set on all desktop
-    QSignalSpy spy(KWindowSystem::self(), static_cast<void (KWindowSystem::*)(WId,NET::Properties,NET::Properties2)>(&KWindowSystem::windowChanged));
+    QSignalSpy spy(KX11Extras::self(), &KX11Extras::windowChanged);
     QVERIFY(spy.isValid());
-    KWindowSystem::setOnAllDesktops(window->winId(), true);
+    KX11Extras::setOnAllDesktops(window->winId(), true);
     QVERIFY(waitForWindow(spy, window->winId(), NET::WMDesktop));
 
     KWindowInfo info2(window->winId(), NET::WMDesktop);
     QVERIFY(info2.isOnCurrentDesktop());
     QVERIFY(info2.onAllDesktops());
     QCOMPARE(info2.desktop(), int(NET::OnAllDesktops));
-    for (int i = 1; i < KWindowSystem::numberOfDesktops(); i++) {
+    for (int i = 1; i < KX11Extras::numberOfDesktops(); i++) {
         QVERIFY(info2.isOnDesktop(i));
     }
 
-    const int desktop = (KWindowSystem::currentDesktop() % KWindowSystem::numberOfDesktops()) + 1;
+    const int desktop = (KX11Extras::currentDesktop() % KX11Extras::numberOfDesktops()) + 1;
     spy.clear();
-    KWindowSystem::setOnDesktop(window->winId(), desktop);
+    KX11Extras::setOnDesktop(window->winId(), desktop);
     QX11Info::getTimestamp();
     QVERIFY(waitForWindow(spy, window->winId(), NET::WMDesktop));
 
@@ -487,7 +494,7 @@ void KWindowInfoX11Test::testDesktop()
     QVERIFY(!info3.isOnCurrentDesktop());
     QVERIFY(!info3.onAllDesktops());
     QCOMPARE(info3.desktop(), desktop);
-    for (int i = 1; i < KWindowSystem::numberOfDesktops(); i++) {
+    for (int i = 1; i < KX11Extras::numberOfDesktops(); i++) {
         if (i == desktop) {
             QVERIFY(info3.isOnDesktop(i));
         } else {
@@ -499,7 +506,8 @@ void KWindowInfoX11Test::testDesktop()
 void KWindowInfoX11Test::testActivities()
 {
     NETRootInfo rootInfo(QX11Info::connection(), NET::Supported | NET::SupportingWMCheck);
-    QSignalSpy spyReal(KWindowSystem::self(), static_cast<void (KWindowSystem::*)(WId,NET::Properties,NET::Properties2)>(&KWindowSystem::windowChanged));
+
+    QSignalSpy spyReal(KX11Extras::self(), &KX11Extras::windowChanged);
     QVERIFY(spyReal.isValid());
 
     KWindowInfo info(window->winId(), NET::Properties(), NET::WM2Activities);
@@ -513,7 +521,7 @@ void KWindowInfoX11Test::testActivities()
     QVERIFY(startingActivities.size() <= 1);
 
     // Window on all activities
-    KWindowSystem::self()->setOnActivities(window->winId(), QStringList());
+    KX11Extras::self()->setOnActivities(window->winId(), QStringList());
 
     QVERIFY(waitForWindow(spyReal, window->winId(), NET::Properties(), NET::WM2Activities));
 
@@ -522,7 +530,7 @@ void KWindowInfoX11Test::testActivities()
     QVERIFY(info2.activities().size() == 0);
 
     // Window on a specific activity
-    KWindowSystem::self()->setOnActivities(window->winId(), QStringList() << "test-activity");
+    KX11Extras::self()->setOnActivities(window->winId(), QStringList() << "test-activity");
 
     QVERIFY(waitForWindow(spyReal, window->winId(), NET::Properties(), NET::WM2Activities));
 
@@ -531,19 +539,19 @@ void KWindowInfoX11Test::testActivities()
     QVERIFY(info3.activities().size() == 1);
     QVERIFY(info3.activities()[0] == "test-activity");
 
-    // Window on a two activities
-    KWindowSystem::self()->setOnActivities(window->winId(), QStringList() << "test-activity" << "test-activity2");
+    // Window on two specific activities
+    KX11Extras::self()->setOnActivities(window->winId(), QStringList{"test-activity", "test-activity2"});
 
     QVERIFY(waitForWindow(spyReal, window->winId(), NET::Properties(), NET::WM2Activities));
 
     KWindowInfo info4(window->winId(), NET::Properties(), NET::WM2Activities);
 
-    QVERIFY(info4.activities().size() == 2);
+    QCOMPARE(info4.activities().size(), 2);
     QVERIFY(info4.activities()[0] == "test-activity");
     QVERIFY(info4.activities()[1] == "test-activity2");
 
     // Window on the starting activity
-    KWindowSystem::self()->setOnActivities(window->winId(), startingActivities);
+    KX11Extras::self()->setOnActivities(window->winId(), startingActivities);
 
     QVERIFY(waitForWindow(spyReal, window->winId(), NET::Properties(), NET::WM2Activities));
 
@@ -559,8 +567,7 @@ void KWindowInfoX11Test::testWindowClass()
     QCOMPARE(info.windowClassClass(), QByteArrayLiteral("kwindowinfox11test"));
 
     // window class needs to be changed using xcb
-    xcb_change_property(QX11Info::connection(), XCB_PROP_MODE_REPLACE, window->winId(),
-                        XCB_ATOM_WM_CLASS, XCB_ATOM_STRING, 8, 7, "foo\0bar");
+    xcb_change_property(QX11Info::connection(), XCB_PROP_MODE_REPLACE, window->winId(), XCB_ATOM_WM_CLASS, XCB_ATOM_STRING, 8, 7, "foo\0bar");
     xcb_flush(QX11Info::connection());
 
     // it's just a property change so we can easily refresh
@@ -578,8 +585,7 @@ void KWindowInfoX11Test::testWindowRole()
 
     // window role needs to be changed using xcb
     KXUtils::Atom atom(QX11Info::connection(), QByteArrayLiteral("WM_WINDOW_ROLE"));
-    xcb_change_property(QX11Info::connection(), XCB_PROP_MODE_REPLACE, window->winId(),
-                        atom, XCB_ATOM_STRING, 8, 3, "bar");
+    xcb_change_property(QX11Info::connection(), XCB_PROP_MODE_REPLACE, window->winId(), atom, XCB_ATOM_STRING, 8, 3, "bar");
     xcb_flush(QX11Info::connection());
 
     // it's just a property change so we can easily refresh
@@ -591,23 +597,33 @@ void KWindowInfoX11Test::testWindowRole()
 
 void KWindowInfoX11Test::testClientMachine()
 {
+    const QByteArray oldHostName = QSysInfo::machineHostName().toLocal8Bit();
+
     KWindowInfo info(window->winId(), NET::Properties(), NET::WM2ClientMachine);
-    QVERIFY(info.clientMachine().isNull());
+    QCOMPARE(info.clientMachine(), oldHostName);
 
     // client machine needs to be set through xcb
-    xcb_change_property(QX11Info::connection(), XCB_PROP_MODE_REPLACE, window->winId(),
-                        XCB_ATOM_WM_CLIENT_MACHINE, XCB_ATOM_STRING, 8, 9, "localhost");
+    const QByteArray newHostName = oldHostName + "2";
+    xcb_change_property(QX11Info::connection(),
+                        XCB_PROP_MODE_REPLACE,
+                        window->winId(),
+                        XCB_ATOM_WM_CLIENT_MACHINE,
+                        XCB_ATOM_STRING,
+                        8,
+                        newHostName.count(),
+                        newHostName.data());
     xcb_flush(QX11Info::connection());
 
     // it's just a property change so we can easily refresh
     QX11Info::getTimestamp();
 
     KWindowInfo info2(window->winId(), NET::Properties(), NET::WM2ClientMachine);
-    QCOMPARE(info2.clientMachine(), QByteArrayLiteral("localhost"));
+    QCOMPARE(info2.clientMachine(), newHostName);
 }
 
 void KWindowInfoX11Test::testName()
 {
+    // clang-format off
     KWindowInfo info(window->winId(), NET::WMName | NET::WMVisibleName | NET::WMIconName | NET::WMVisibleIconName | NET::WMState | NET::XAWMState);
     QCOMPARE(info.name(),                     QStringLiteral("kwindowinfox11test"));
     QCOMPARE(info.visibleName(),              QStringLiteral("kwindowinfox11test"));
@@ -649,6 +665,7 @@ void KWindowInfoX11Test::testName()
     QCOMPARE(info3.iconName(),                 QStringLiteral("foobar"));
     QCOMPARE(info3.visibleIconName(),          QStringLiteral("foobar"));
     QCOMPARE(info3.visibleIconNameWithState(), QStringLiteral("(foobar)"));
+    // clang-format on
 }
 
 void KWindowInfoX11Test::testTransientFor()
@@ -657,14 +674,13 @@ void KWindowInfoX11Test::testTransientFor()
     QCOMPARE(info.transientFor(), WId(0));
 
     // let's create a second window
-    QScopedPointer<QWidget> window2(new QWidget());
+    std::unique_ptr<QWidget> window2(new QWidget());
     window2->show();
-    QVERIFY(QTest::qWaitForWindowExposed(window2.data()));
+    QVERIFY(QTest::qWaitForWindowExposed(window2.get()));
 
     // update the transient for of window1 to window2
     const uint32_t id = window2->winId();
-    xcb_change_property(QX11Info::connection(), XCB_PROP_MODE_REPLACE, window->winId(),
-                        XCB_ATOM_WM_TRANSIENT_FOR, XCB_ATOM_WINDOW, 32, 1, &id);
+    xcb_change_property(QX11Info::connection(), XCB_PROP_MODE_REPLACE, window->winId(), XCB_ATOM_WM_TRANSIENT_FOR, XCB_ATOM_WINDOW, 32, 1, &id);
     xcb_flush(QX11Info::connection());
 
     KWindowInfo info2(window->winId(), NET::Properties(), NET::WM2TransientFor);
@@ -673,11 +689,24 @@ void KWindowInfoX11Test::testTransientFor()
 
 void KWindowInfoX11Test::testGroupLeader()
 {
-    KWindowInfo info(window->winId(), NET::Properties(), NET::WM2GroupLeader);
-    QCOMPARE(info.groupLeader(), WId(0));
+    // WM_CLIENT_LEADER is set by default
+    KWindowInfo info1(window->winId(), NET::Properties(), NET::WM2GroupLeader);
+    QVERIFY(info1.groupLeader() != XCB_WINDOW_NONE);
 
-    // TODO: here we should try to set a group leader and re-read it
-    // this needs setting and parsing the WMHints
+    xcb_connection_t *connection = QX11Info::connection();
+    xcb_window_t rootWindow = QX11Info::appRootWindow();
+
+    xcb_window_t leader = xcb_generate_id(connection);
+    xcb_create_window(connection, XCB_COPY_FROM_PARENT, leader, rootWindow, 0, 0, 1, 1, 0, XCB_WINDOW_CLASS_INPUT_OUTPUT, XCB_COPY_FROM_PARENT, 0, nullptr);
+
+    xcb_icccm_wm_hints_t hints = {};
+    hints.flags = XCB_ICCCM_WM_HINT_WINDOW_GROUP;
+    hints.window_group = leader;
+    xcb_icccm_set_wm_hints(connection, leader, &hints);
+    xcb_icccm_set_wm_hints(connection, window->winId(), &hints);
+
+    KWindowInfo info2(window->winId(), NET::Properties(), NET::WM2GroupLeader);
+    QCOMPARE(info2.groupLeader(), leader);
 }
 
 void KWindowInfoX11Test::testExtendedStrut()
@@ -697,7 +726,7 @@ void KWindowInfoX11Test::testExtendedStrut()
     QCOMPARE(strut.top_start, 0);
     QCOMPARE(strut.top_width, 0);
 
-    KWindowSystem::setExtendedStrut(window->winId(), 10, 20, 30, 40, 5, 15, 25, 35, 2, 12, 22, 32);
+    KX11Extras::setExtendedStrut(window->winId(), 10, 20, 30, 40, 5, 15, 25, 35, 2, 12, 22, 32);
 
     // it's just an xprop, so one roundtrip is good enough
     QX11Info::getTimestamp();
@@ -724,12 +753,12 @@ void KWindowInfoX11Test::testGeometry()
     QCOMPARE(info.geometry().size(), window->geometry().size());
     QCOMPARE(info.frameGeometry().size(), window->frameGeometry().size());
 
-    QSignalSpy spy(KWindowSystem::self(), static_cast<void (KWindowSystem::*)(WId,NET::Properties,NET::Properties2)>(&KWindowSystem::windowChanged));
+    QSignalSpy spy(KX11Extras::self(), &KX11Extras::windowChanged);
     QVERIFY(spy.isValid());
 
     // this is tricky, KWin is smart and doesn't allow all geometries we pass in
     // setting to center of screen should work, though
-    QRect geo(window->windowHandle()->screen()->geometry().center() - QPoint(window->width()/2-5, window->height()/2-5),
+    QRect geo(window->windowHandle()->screen()->geometry().center() - QPoint(window->width() / 2 - 5, window->height() / 2 - 5),
               window->size() + QSize(10, 10));
     window->setGeometry(geo);
     waitForWindow(spy, window->winId(), NET::WMGeometry);
@@ -746,7 +775,7 @@ void KWindowInfoX11Test::testDesktopFileName()
     QVERIFY(info.valid());
     QCOMPARE(info.desktopFileName(), QByteArray());
 
-    QSignalSpy spy(KWindowSystem::self(), static_cast<void (KWindowSystem::*)(WId,NET::Properties,NET::Properties2)>(&KWindowSystem::windowChanged));
+    QSignalSpy spy(KX11Extras::self(), &KX11Extras::windowChanged);
     QVERIFY(spy.isValid());
 
     // create a NETWinInfo to set the desktop file name
